package org.bitbucket.bradleysmithllc.etlunit;

import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.regexp.TestExpression;
import org.bitbucket.bradleysmithllc.etlunit.regexp.TestSpecificationExpression;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class PatternClassDirector extends NullClassDirector {
	private final List<NamePattern> matchList = new ArrayList<NamePattern>();

	private final class NamePattern {
		private final Pattern className;
		private final Pattern methodName;
		private final Pattern operName;

		private final String classNameText;
		private final String methodNameText;
		private final String operNameText;

		private NamePattern(String className, String methodName, String operName) {
			classNameText = className;
			methodNameText = methodName;
			operNameText = operName;

			this.className = className != null ? Pattern.compile(className, Pattern.CASE_INSENSITIVE) : null;
			this.methodName = methodName != null ? Pattern.compile(methodName, Pattern.CASE_INSENSITIVE) : null;
			this.operName = operName != null ? Pattern.compile(operName, Pattern.CASE_INSENSITIVE) : null;
		}

		boolean classMatches(ETLTestClass cls) {
			return className == null ? true : className.matcher(cls.getQualifiedName()).find();
		}

		boolean methodMatches(ETLTestMethod cls) {
			return classMatches(cls.getTestClass()) &&
					(methodName == null ? true : methodName.matcher(cls.getName()).find());
		}

		boolean operationMatches(ETLTestOperation ope) {
			// we don't want to cancel any before or after methods, so only filter the operations if this method accepts
			// If the method does not accept, that means it did NOT come through the method accept.
			return (operName == null ? true : operName.matcher(ope.getOperationName()).find());
		}

		@Override
		public String toString() {
			return "NamePattern{" +
					"classNameText='" + classNameText + '\'' +
					(methodNameText != null ? (", methodNameText='" + methodNameText + '\'') : "") +
					(operNameText != null ? (", operNameText='" + operNameText + '\'') : "") +
					'}';
		}
	}

	public PatternClassDirector(TestExpression tex) {
		while (tex.hasNext()) {
			TestSpecificationExpression tspec = tex.getTestSpecification();

			if (tspec.matches()) {
				matchList.add(
						new NamePattern(
								tspec.getTestClassPattern(),
								tspec.hasTestNamePattern() ? tspec.getTestNamePattern() : null,
								tspec.hasTestOperationPattern() ? tspec.getTestOperationPattern() : null
						)
				);
			}
		}
	}

	public PatternClassDirector(String classPattern, String methodPattern, String operationPattern) {
		matchList.add(
			new NamePattern(
				classPattern,
				methodPattern,
				operationPattern
			)
		);
	}

	@Override
	public response_code accept(ETLTestClass cl) {
		for (NamePattern np : matchList) {
			if (np.classMatches(cl)) {
				return response_code.accept;
			}
		}

		return response_code.reject;
	}

	@Override
	public response_code accept(ETLTestMethod mt) {
		for (NamePattern np : matchList) {
			if (np.methodMatches(mt)) {
				return response_code.accept;
			}
		}

		return response_code.reject;
	}

	@Override
	public response_code accept(ETLTestOperation op) {
		for (NamePattern np : matchList) {
			if (np.operationMatches(op)) {
				return response_code.accept;
			}
		}

		return response_code.reject;
	}
}